﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace SimpleTextEditor
{
    public partial class SimpleTextEditorForm : Form
    {
        //フォームのタイトル
        private const string TitleName = " - 簡易テキストエディタ";

        //フォームのタイトルに表示する、未保存時のファイル名
        private const string NewFileName = "無題";

        //編集中ファイルのパス
        private string _editFilePath;

        //コンストラクタ
        public SimpleTextEditorForm()
        {
            InitializeComponent();
            InitTitleAndPath();
        }

        //編集中ファイルのパスとフォームのタイトルを初期化する
        private void InitTitleAndPath()
        {
            _editFilePath = string.Empty;
            this.Text = NewFileName + TitleName;
        }

        //編集中ファイルのパスとフォームのタイトルを設定する
        private void SetTitleAndPath(string filePath)
        {
            _editFilePath = filePath;
            this.Text = Path.GetFileName(filePath) + TitleName;
        }

        //「新規」メニュー項目イベント処理
        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //初期化を行う
            textBox1.Clear();
            InitTitleAndPath();
        }

        //「開く」メニュー項目イベント処理
        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                //OpenFileDialogで指定されたファイルを開く
                LoadFile(openFileDialog1.FileName);
            }
        }

        //「上書き保存」メニュー項目イベント処理
        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //ファイルを開いていない or 新規作成の場合、「名前を付けて保存」を行う
            if (string.IsNullOrEmpty(_editFilePath))
            {
                if (saveFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    SaveFile(saveFileDialog1.FileName);
                }
            }
            else
            {
                //上書きする対象のファイルが存在するかチェックを行った後、ファイルを上書き保存
                if (File.Exists(_editFilePath))
                {
                    SaveFile(_editFilePath);
                }
                else
                {
                    MessageBox.Show("ファイルが存在しません。", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        //「名前を付けて保存」メニュー項目イベント処理
        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                //SaveFileDialogで指定したファイル名・場所にファイルを保存する
                SaveFile(saveFileDialog1.FileName);
            }
        }

        //「終了」メニュー項目イベント処理
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        //ファイル読み込み処理
        private void LoadFile(string filePath)
        {
            try
            {
                using (StreamReader reader = new StreamReader(filePath))
                {                    
                    textBox1.Text = reader.ReadToEnd();
                    SetTitleAndPath(filePath);
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //ファイル書き込み処理
        private void SaveFile(string filePath)
        {
            try
            {
                using (StreamWriter writer = new StreamWriter(filePath))
                {
                    writer.Write(textBox1.Text);
                    SetTitleAndPath(filePath);
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //フォームにファイルがドラッグされた時のイベント処理
        private void SimpleTextEditorForm_DragEnter(object sender, DragEventArgs e)
        {
            //単一のファイルがドラッグされた場合のみ、ドロップを許可する
            string[] fileNames = (string[])e.Data.GetData(DataFormats.FileDrop, false);
            if (e.Data.GetDataPresent(DataFormats.FileDrop) && fileNames.Length == 1)
            {
                e.Effect = DragDropEffects.All;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        //フォームにファイルがドロップされた時のイベント処理
        private void SimpleTextEditorForm_DragDrop(object sender, DragEventArgs e)
        {
            //ドラッグ＆ドロップされたファイルをオープンする
            string[] fileNames = (string[])e.Data.GetData(DataFormats.FileDrop, false);
            LoadFile(fileNames.First());
        }
    }
}
