﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace Janken
{
  public partial class MainForm : Form
  {
    private int userWinCount = 0;
    private int userLossCount = 0;
    private int userDrawCount = 0;
    private Dictionary<int, Image>
      userImageDictionary = null;
    private Dictionary<int, Image>
      enemyImageDictionary = null;
    private Dictionary<int, Dictionary<int, int>>
      judgmentDictionary = null;
    private Dictionary<string, IEnemy>
      enemyDictionary = null;
    private IEnemy enemy = null;

    public MainForm()
    {
      InitializeComponent();

      Image rockImage =
        Properties.Resources.RockImage;
      Image scissorsImage =
        Properties.Resources.ScissorsImage;
      Image paperImage =
        Properties.Resources.PaperImage;
      Image enemyRockImage =
        (Image)rockImage.Clone();
      Image enemyScissorsImage =
        (Image)scissorsImage.Clone();
      Image enemyPaperImage =
        (Image)paperImage.Clone();
      enemyRockImage.RotateFlip
        (RotateFlipType.Rotate180FlipNone);
      enemyScissorsImage.RotateFlip
        (RotateFlipType.Rotate180FlipNone);
      enemyPaperImage.RotateFlip
        (RotateFlipType.Rotate180FlipNone);

      userImageDictionary =
        new Dictionary<int, Image>();
      userImageDictionary.Add
        (JankenConst.Rock, rockImage);
      userImageDictionary.Add
        (JankenConst.Scissors, scissorsImage);
      userImageDictionary.Add
        (JankenConst.Paper, paperImage);
      enemyImageDictionary =
        new Dictionary<int, Image>();
      enemyImageDictionary.Add
        (JankenConst.Rock, enemyRockImage);
      enemyImageDictionary.Add
        (JankenConst.Scissors, enemyScissorsImage);
      enemyImageDictionary.Add
        (JankenConst.Paper, enemyPaperImage);

      Dictionary<int, int> userRockDictionary =
        new Dictionary<int, int>();
      Dictionary<int, int> userScissorsDictionary =
        new Dictionary<int, int>();
      Dictionary<int, int> userPaperDictionary =
        new Dictionary<int, int>();
      userRockDictionary.Add
        (JankenConst.Rock, JankenConst.Draw);
      userRockDictionary.Add
        (JankenConst.Scissors, JankenConst.UserWin);
      userRockDictionary.Add
        (JankenConst.Paper, JankenConst.EnemyWin);
      userScissorsDictionary.Add
        (JankenConst.Rock, JankenConst.EnemyWin);
      userScissorsDictionary.Add
        (JankenConst.Scissors, JankenConst.Draw);
      userScissorsDictionary.Add
        (JankenConst.Paper, JankenConst.UserWin);
      userPaperDictionary.Add
        (JankenConst.Rock, JankenConst.UserWin);
      userPaperDictionary.Add
        (JankenConst.Scissors, JankenConst.EnemyWin);
      userPaperDictionary.Add
        (JankenConst.Paper, JankenConst.Draw);

      judgmentDictionary = new Dictionary
        <int, Dictionary<int, int>>();
      judgmentDictionary.Add
        (JankenConst.Rock, userRockDictionary);
      judgmentDictionary.Add(JankenConst.Scissors,
        userScissorsDictionary);
      judgmentDictionary.Add
        (JankenConst.Paper, userPaperDictionary);

      IEnemy enemyDefault = new EnemyDefault();
      IEnemy enemyPaper = new EnemyPaper();
      IEnemy enemyScissors = new EnemyScissors();

      enemyDictionary =
        new Dictionary<string, IEnemy>();
      enemyDictionary.Add
        (enemyDefault.Name, enemyDefault);
      enemyDictionary.Add
        (enemyPaper.Name, enemyPaper);
      enemyDictionary.Add
        (enemyScissors.Name, enemyScissors);

      foreach (IEnemy enemyValue
        in enemyDictionary.Values)
      {
        enemyListBox.Items.Add(enemyValue.Name);
      }
      enemyListBox.SetSelected(0, true);
    }

    private void enemyListBox_SelectedIndexChanged(
      object sender, EventArgs e)
    {

      enemy = enemyDictionary[enemyListBox.Text];
      userWinCount = 0;
      userLossCount = 0;
      userDrawCount = 0;
      myPictureBox.Image =
        userImageDictionary[JankenConst.Rock];
      enemyPictureBox.Image =
        enemyImageDictionary[JankenConst.Rock];
      enemyNameLabel.Text = enemy.Name;
      userWinsLabel.Text = string.Empty;
      enemyWinsLabel.Text = string.Empty;
      userWinCountLabel.Text =
        userWinCount.ToString();
      userLossCountLabel.Text =
        userLossCount.ToString();
      userDrawCountLabel.Text =
        userDrawCount.ToString();
    }

    private void rockButton_Click(object sender,
      EventArgs e)
    {
      janken(JankenConst.Rock);
    }

    private void scissorsButton_Click(object
      sender, EventArgs e)
    {
      janken(JankenConst.Scissors);
    }

    private void paperButton_Click(object
      sender, EventArgs e)
    {
      janken(JankenConst.Paper);
    }

    private void janken(int userValue)
    {
      int enemyValue = enemy.Janken(userValue);
      myPictureBox.Image =
        userImageDictionary[userValue];
      enemyPictureBox.Image =
        enemyImageDictionary[enemyValue];
      int judgmentValue =
        judgmentDictionary[userValue][enemyValue];
      displayResults(judgmentValue);
    }

    private void displayResults(int
      judgmentValue)
    {
      switch (judgmentValue)
      {
        case JankenConst.UserWin:
          userWinCount++;
          userWinsLabel.Text =
            Properties.Resources.WinString;
          enemyWinsLabel.Text =
            Properties.Resources.LossString;
          break;
        case JankenConst.EnemyWin:
          userLossCount++;
          userWinsLabel.Text =
            Properties.Resources.LossString;
          enemyWinsLabel.Text =
            Properties.Resources.WinString;
          break;
        default:
          userDrawCount++;
          userWinsLabel.Text =
            Properties.Resources.DrawString;
          enemyWinsLabel.Text =
            Properties.Resources.DrawString;
          break;
      }
      userWinCountLabel.Text =
        userWinCount.ToString();
      userLossCountLabel.Text =
        userLossCount.ToString();
      userDrawCountLabel.Text =
        userDrawCount.ToString();
    }
  }
}
